﻿using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows.Forms;

namespace SRD_Interface
{
    public partial class MainForm : Form
    {
        private ConfigData config = new ConfigData();
        private static string token = "95185..."; //신청시 발급받은 token
        private static string accessToken = "Bearer " + token;  // 무기명 인증 토큰
        private Dictionary<string, string> orderDetailsBlockList = new Dictionary<string, string>();

        public MainForm()
        {
            InitializeComponent();
        }
        private void Form1_Load(object sender, EventArgs e)
        {
            // 초기 세팅
            cbCommandList.SelectedIndex = 0;
            string path = Application.StartupPath + @"\Config.json";
            string users = string.Empty;
            JObject configData = new JObject(
                new JProperty("IP", "127.0.0.1"),  // 192.168.0.174
                new JProperty("PORT", "8088")
                );

            // 파일 생성 후 쓰기
            File.WriteAllText(path, configData.ToString());

            // 로그 기록
            tbLogBox.Text = configData.ToString();

            // Json 파일 읽기
            using (StreamReader file = File.OpenText(path))
            using (JsonTextReader reader = new JsonTextReader(file))
            {
                JObject json = (JObject)JToken.ReadFrom(reader);

                // Config
                config.IP = json["IP"].ToString();
                config.PORT = (int)json["PORT"];

                // 로그 기록
                string str = $"IP : {config.IP}\r\n" +
                             $"PORT : {config.PORT}\r\n";
                tbLogBox.Text += "\r\n\r\n" + str;
            }

            // Location
            panel_VehicleDetails_Send.Location = new System.Drawing.Point(6, 68);
            panel_reLocConfirm_Send.Location = new System.Drawing.Point(6, 68);
            panel_EnergyRecharged_Send.Location = new System.Drawing.Point(6, 68);

            // Size
            panel_VehicleDetails_Send.Size = new System.Drawing.Size(807, 476);
            panel_reLocConfirm_Send.Size = new System.Drawing.Size(807, 476);
            panel_EnergyRecharged_Send.Size = new System.Drawing.Size(807, 476);
        }

        // 이벤트
        private void btnLogClear_Click(object sender, EventArgs e)
        {
            tbLogBox.Clear();
        }
        private void btnSend_Click(object sender, EventArgs e)
        {
            // 전송 데이터 설정
            string uri = "http://192.168.101.95:9982";
            string method = "GET";
            string sendData = "";
            if (cbCommandList.SelectedIndex == 0)
            {
                if (tbVehicleDetailsName_Send.Text.Trim() == "")
                {
                    MessageBox.Show("필수 항목을 기입해주세요.", "알림", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                tbVehicleDetailsName_Send.ReadOnly = true;

                // uri 추가
                uri += string.Format("/vehicleDetails/{0}", tbVehicleDetailsName_Send.Text);
            }
            else if (cbCommandList.SelectedIndex == 1)
            {
                if (tbReLocConfirmVehicle1.Text.Trim() == "" && tbReLocConfirmVehicle2.Text.Trim() == "")
                {
                    MessageBox.Show("필수 항목을 기입해주세요.", "알림", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // Object 생성
                JArray addJArray = new JArray(tbReLocConfirmVehicle1.Text, tbReLocConfirmVehicle2.Text);
                JObject addJObject = new JObject(
                    new JProperty("vehicles", addJArray));

                // uri 추가, 전송 값 생성
                uri += "/reLocConfirm";
                sendData = string.Format("{0}", addJObject);
                method = "POST";
            }
            else if (cbCommandList.SelectedIndex == 5)
            {
                if (tbEnergyRechargeLevelChangeName_Send.Text.Trim() == "" || tbEnergyRechargeLevelChangeValue_Send.Text.Trim() == "")
                {
                    MessageBox.Show("필수 항목을 기입해주세요.", "알림", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                // uri 추가
                uri += string.Format("/vehicle/{0}/energyLevelSufficientlyRecharged?newValue={1}", tbEnergyRechargeLevelChangeName_Send.Text, tbEnergyRechargeLevelChangeValue_Send.Text);
                method = "PUT";
            }

            // HttpWebRequest 설정
            HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(uri);
            httpWebRequest.Method = method;
            httpWebRequest.ContentType = "application/json; charset=utf-8";
            httpWebRequest.Headers.Add("Authorization", accessToken);
            httpWebRequest.KeepAlive = true;
            httpWebRequest.Timeout = 10000;

            // 보낼 데이터를 byteArray로 바꿔준다.
            byte[] byteArray = Encoding.UTF8.GetBytes(sendData);

            if (httpWebRequest.Method != "GET" && sendData != "")
            {
                // 요청 Data를 쓰는 데 사용할 Stream 개체를 가져온다.
                using (Stream dataStream = httpWebRequest.GetRequestStream())
                {
                    dataStream.Write(byteArray, 0, byteArray.Length);
                    dataStream.Close();
                }
            }

            // 응답 받기
            try
            {
                using (HttpWebResponse resp = (HttpWebResponse)httpWebRequest.GetResponse())
                {
                    HttpStatusCode status = resp.StatusCode;
                    Stream respStream = resp.GetResponseStream();
                    using (StreamReader streamReader = new StreamReader(respStream))
                    {
                        string responseText = "";
                        try
                        {
                            responseText = streamReader.ReadToEnd();  // POST 방식으로 보내니 문제생김
                        }
                        catch (IOException ex)
                        {
                            tbLogBox.Text += "\r\n" + ex;
                            return;
                        }

                        if (status == HttpStatusCode.OK)
                        {
                            // 파싱 작업까지 진행
                            JObject parseObject = JObject.Parse(responseText);
                        }

                        // LOG 창에 기록
                        tbLogBox.Text += "\r\n" + responseText;
                        tbVehicleDetailsName_Send.ReadOnly = false;
                    }
                }
            }
            catch (WebException)
            {
                MessageBox.Show("수신 대기 시간이 초과되었습니다.", "알림", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }
        }
        private void cbCommandList_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (rb_SendData.Checked) rb_SendData_CheckedChanged(null, null);
        }
        private void rb_SendData_CheckedChanged(object sender, EventArgs e)
        {
            panel_VehicleDetails_Send.Visible = false;
            panel_reLocConfirm_Send.Visible = false;
            panel_EnergyRecharged_Send.Visible = false;

            if (cbCommandList.SelectedIndex == 0) panel_VehicleDetails_Send.Visible = true;
            else if (cbCommandList.SelectedIndex == 1) panel_reLocConfirm_Send.Visible = true;
            else if (cbCommandList.SelectedIndex == 5) panel_EnergyRecharged_Send.Visible = true;
        }

    }
}
